/*
 * Copyright (c) 2012 IBM and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   E.D.Willink - complete code for fragmentary Wiki example.
 *
 */
package org.eclipse.uml2.uml.examples.standalone;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.resource.UMLResource;

/**
 * This class provides a working example of
 * http://wiki.eclipse.org/MDT/UML2/Getting_Started_with_UML2.
 */
public class GettingStartedExample extends GettingStartedUtils
{
    @SuppressWarnings("unused")
	public static void main(String[] args) {
    	if (args.length != 1) {
    		err("Usage: <results-folder>");
    		return;
    	}
        //
        //	Init
        //
    	ResourceSet resourceSet = new ResourceSetImpl();
    	init(resourceSet);
        //
        //	Create
        //
    	Model epo2Model = createExample();
        //
        //	Save
        //
        URI folderURI = URI.createFileURI(args[0]);
		URI fileURI = folderURI.appendSegment("ExtendedPO2").appendFileExtension(UMLResource.FILE_EXTENSION);
		save(resourceSet, epo2Model, fileURI);     
        //
        //	Load
        //
    	ResourceSet reloadResourceSet = new ResourceSetImpl();
    	init(reloadResourceSet);
		Model reloadedModel = (Model) load(reloadResourceSet, fileURI);     
    }
    
    public static Model createExample() {
        //
        //	Models and Packages
        //
        Model epo2Model = createModel("epo2");
//      org.eclipse.uml2.uml.Package barPackage = createPackage(fooPackage, "bar");
        //
        //	PrimitiveTypes
        //
        PrimitiveType datePrimitiveType = createPrimitiveType(epo2Model, "Date");
        PrimitiveType intPrimitiveType = createPrimitiveType(epo2Model, "int");
        PrimitiveType skuPrimitiveType = createPrimitiveType(epo2Model, "SKU");
        PrimitiveType stringPrimitiveType = createPrimitiveType(epo2Model, "String");
        //
        //	Enumerations
        //
        Enumeration orderStatusEnumeration = createEnumeration(epo2Model, "OrderStatus");
        //
        //	EnumerationLiterals
        //
        createEnumerationLiteral(orderStatusEnumeration, "BackOrder");
        createEnumerationLiteral(orderStatusEnumeration, "Complete");
        createEnumerationLiteral(orderStatusEnumeration, "Pending");
        //
        //	Classes
        //
        org.eclipse.uml2.uml.Class addressClass = createClass(epo2Model, "Address", false);
        org.eclipse.uml2.uml.Class customerClass = createClass(epo2Model, "Customer", false);
        org.eclipse.uml2.uml.Class globalAddressClass = createClass(epo2Model, "GlobalAddress", false);
        org.eclipse.uml2.uml.Class globalLocationClass = createClass(epo2Model, "GlobalLocation", false);
        org.eclipse.uml2.uml.Class itemClass = createClass(epo2Model, "Item", false);
        org.eclipse.uml2.uml.Class purchaseOrderClass = createClass(epo2Model, "PurchaseOrder", false);
        org.eclipse.uml2.uml.Class supplierClass = createClass(epo2Model, "Supplier", false);
        org.eclipse.uml2.uml.Class usAddressClass = createClass(epo2Model, "USAddress", false);
        //
        //	Generalizations
        //
    	createGeneralization(usAddressClass, addressClass);
    	createGeneralization(globalAddressClass, addressClass);
    	createGeneralization(globalAddressClass, globalLocationClass);
        //
        //	Attributes
        //
        createAttribute(addressClass, "country", stringPrimitiveType, 0, 1); 
        createAttribute(addressClass, "name", stringPrimitiveType, 0, 1); 
        createAttribute(customerClass, "customerID", intPrimitiveType, 0, 1); 
        createAttribute(globalAddressClass, "location", stringPrimitiveType, 0, 1); 
        createAttribute(globalLocationClass, "countryCode", intPrimitiveType, 0, 1); 
        createAttribute(itemClass, "productName", stringPrimitiveType, 0, 1); 
        createAttribute(itemClass, "quantity", intPrimitiveType, 0, 1); 
        createAttribute(itemClass, "USPrice", intPrimitiveType, 0, 1); 
        createAttribute(itemClass, "comment", stringPrimitiveType, 0, 1); 
        createAttribute(itemClass, "shipDate", datePrimitiveType, 0, 1); 
        createAttribute(itemClass, "partNum", skuPrimitiveType, 0, 1); 
        createAttribute(purchaseOrderClass, "comment", stringPrimitiveType, 0, 1); 
        createAttribute(purchaseOrderClass, "orderDate", datePrimitiveType, 0, 1); 
        createAttribute(purchaseOrderClass, "status", orderStatusEnumeration, 0, 1); 
        createAttribute(purchaseOrderClass, "totalAmount", intPrimitiveType, 0, 1); 
        createAttribute(supplierClass, "name", stringPrimitiveType, 0, 1); 
        createAttribute(usAddressClass, "street", stringPrimitiveType, 0, 1); 
        createAttribute(usAddressClass, "city", stringPrimitiveType, 0, 1); 
        createAttribute(usAddressClass, "state", stringPrimitiveType, 0, 1); 
        createAttribute(usAddressClass, "zip", intPrimitiveType, 0, 1); 
        //
        //	Associations
        //
        createAssociation(supplierClass, true, AggregationKind.COMPOSITE_LITERAL, "orders", 0, LiteralUnlimitedNatural.UNLIMITED,
                purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 1, 1);
        createAssociation(supplierClass, true, AggregationKind.COMPOSITE_LITERAL, "customers", 0, LiteralUnlimitedNatural.UNLIMITED,
                customerClass, false, AggregationKind.NONE_LITERAL, "", 1, 1);
        createAssociation(purchaseOrderClass, true, AggregationKind.COMPOSITE_LITERAL, "billTo", 1, 1,
        		addressClass, false, AggregationKind.NONE_LITERAL, "", 1, 1);
        createAssociation(purchaseOrderClass, true, AggregationKind.COMPOSITE_LITERAL, "shipTo", 0, 1,
        		addressClass, false, AggregationKind.NONE_LITERAL, "", 1, 1);
        createAssociation(purchaseOrderClass, true, AggregationKind.COMPOSITE_LITERAL, "items", 0, LiteralUnlimitedNatural.UNLIMITED,
        		itemClass, false, AggregationKind.NONE_LITERAL, "order", 1, 1);

        createAssociation(supplierClass, true, AggregationKind.NONE_LITERAL, "pendingOrders", 0, LiteralUnlimitedNatural.UNLIMITED,
                purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 0, 1);
        createAssociation(supplierClass, true, AggregationKind.NONE_LITERAL, "shippedOrders", 0, LiteralUnlimitedNatural.UNLIMITED,
                purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 0, 1);
        createAssociation(purchaseOrderClass, true, AggregationKind.NONE_LITERAL, "previousOrder", 0, 1,
                purchaseOrderClass, false, AggregationKind.NONE_LITERAL, "", 1, 1);
        
        return epo2Model;
    }
    
}
