/*
 * Copyright (c) 2012 IBM and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   E.D.Willink - complete code for fragmentary Wiki example.
 *
 */
package org.eclipse.uml2.uml.examples.standalone;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.resource.UMLResource;

public class GettingStartedTestCase extends TestCase
{
	public void testGettingStarted() {
		URL modelsURL = getClass().getResource("models");
		String modelsDir = modelsURL.toString().substring(6);
		GettingStartedExample.main(new String[] {modelsDir});
		checkFile("models/ExtendedPO2.3.0.0.uml", "models/ExtendedPO2.uml", 45);
		if (!"org.eclipse.uml2.uml_3_0_0".equals(UMLResource.UML_CONTENT_TYPE_IDENTIFIER)) {
			checkFile("models/ExtendedPO2.4.0.0.uml", "models/ExtendedPO2.uml", 45);
		}
	}

	public void testIntroductionToProfiles() {
		URL modelsURL = getClass().getResource("models");
		String modelsDir = modelsURL.toString().substring(6);
		IntroductionToUMLProfilesExample.main(new String[] {modelsDir});
		checkFile("models/Ecore.profile.3.0.0.uml", "models/Ecore.profile.uml", 33);
		if (!"org.eclipse.uml2.uml_3_0_0".equals(UMLResource.UML_CONTENT_TYPE_IDENTIFIER)) {
			checkFile("models/Ecore.profile.4.0.0.uml", "models/Ecore.profile.uml", 33);
		}
		// FIXME Test profiled save  
	}

	private void checkFile(String referenceFileName, String generatedFileName, int namedElementCount) {
		ResourceSet generatedResourceSet = new ResourceSetImpl();
		ResourceSet referenceResourceSet = new ResourceSetImpl();
		GettingStartedUtils.init(generatedResourceSet);
		GettingStartedUtils.init(referenceResourceSet);
		URI generatedURI = URI.createURI(getClass().getResource(generatedFileName).toString());
		URI referenceURI = URI.createURI(getClass().getResource(referenceFileName).toString());
		org.eclipse.uml2.uml.Package generatedModel = GettingStartedUtils.load(generatedResourceSet, generatedURI);
		org.eclipse.uml2.uml.Package referenceModel = GettingStartedUtils.load(referenceResourceSet, referenceURI);
		Map<String,EClass> generatedMetric = comparisonMetric(generatedModel);
		assertEquals(namedElementCount, generatedMetric.size());
		Map<String,EClass> referenceMetric = comparisonMetric(referenceModel);
		assertEquals(generatedMetric, referenceMetric);
	}
	
	/**
	 * The names are well-behaved, so a simple name to EClass map can detect numerous failures
	 * without incurring dependencies on EMF Compare.
	 */
	public Map<String,EClass> comparisonMetric(EObject rootObject) {
		Map<String,EClass> metric = new HashMap<String,EClass>();
		for (TreeIterator<EObject> tit = rootObject.eAllContents(); tit.hasNext(); ) {
			EObject eObject = tit.next();
			if (eObject instanceof NamedElement) {
				metric.put(((NamedElement)eObject).getName(), eObject.eClass());
			}
		}
		return metric;
	}
}
