/*
 * Copyright (c) 2012 IBM and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   E.D.Willink - complete code for fragmentary Wiki example.
 *
 */
package org.eclipse.uml2.uml.examples.standalone;

import java.io.IOException;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.Extension;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.resource.UMLResource;

/**
 * This class provides the utility methods described in
 * http://wiki.eclipse.org/MDT/UML2/Getting_Started_with_UML2
 * and http://wiki.eclipse.org/MDT/UML2/Introduction_to_UML2_Profiles
 */
public class GettingStartedUtils
{	
    public static boolean DEBUG = true;

    public static void applyProfile(org.eclipse.uml2.uml.Package package_, Profile profile) {
        package_.applyProfile(profile);

        out("Profile '" + profile.getQualifiedName() + "' applied to package '" + package_.getQualifiedName() + "'.");
    }   

    public static void applyStereotype(NamedElement namedElement, Stereotype stereotype) {
        namedElement.applyStereotype(stereotype);

        out("Stereotype '" + stereotype.getQualifiedName() + "' applied to element '" + namedElement.getQualifiedName() + "'.");
    }    
    
    public static Association createAssociation(Type type1, boolean end1IsNavigable, AggregationKind end1Aggregation,
            String end1Name, int end1LowerBound, int end1UpperBound,
            Type type2, boolean end2IsNavigable, AggregationKind end2Aggregation,
            String end2Name, int end2LowerBound, int end2UpperBound) {

        Association association = type1.createAssociation(end1IsNavigable, end1Aggregation, end1Name, end1LowerBound, end1UpperBound,
            type2, end2IsNavigable, end2Aggregation, end2Name, end2LowerBound, end2UpperBound);

        StringBuffer sb = new StringBuffer();

        sb.append("Association ");

        if (null == end1Name || 0 == end1Name.length()) {
            sb.append('{');
            sb.append(type1.getQualifiedName());
            sb.append('}');
        } else {
            sb.append("'");
            sb.append(type1.getQualifiedName());
            sb.append(NamedElement.SEPARATOR);
            sb.append(end1Name);
            sb.append("'");
        }

        sb.append(" [");
        sb.append(end1LowerBound);
        sb.append("..");
        sb.append(LiteralUnlimitedNatural.UNLIMITED == end1UpperBound ? "*" : String.valueOf(end1UpperBound));
        sb.append("] ");

        sb.append(end2IsNavigable ? '<' : '-');
        sb.append('-');
        sb.append(end1IsNavigable ? '>' : '-');
        sb.append(' ');

        if (null == end2Name || 0 == end2Name.length()) {
            sb.append('{');
            sb.append(type2.getQualifiedName());
            sb.append('}');
        } else {
            sb.append("'");
            sb.append(type2.getQualifiedName());
            sb.append(NamedElement.SEPARATOR);
            sb.append(end2Name);
            sb.append("'");
        }

        sb.append(" [");
        sb.append(end2LowerBound);
        sb.append("..");
        sb.append(LiteralUnlimitedNatural.UNLIMITED == end2UpperBound ? "*" : String.valueOf(end2UpperBound));
        sb.append("]");

        sb.append(" created.");

        out(sb.toString());

        return association;
    }
    
    public static Property createAttribute(org.eclipse.uml2.uml.Class class_, String name, Type type, int lowerBound, int upperBound) {
        Property attribute = class_.createOwnedAttribute(name, type, lowerBound, upperBound);

        StringBuffer sb = new StringBuffer();

        sb.append("Attribute '");

        sb.append(attribute.getQualifiedName());

        sb.append("' : ");

        sb.append(type.getQualifiedName());

        sb.append(" [");
        sb.append(lowerBound);
        sb.append("..");
        sb.append(LiteralUnlimitedNatural.UNLIMITED == upperBound ? "*" : String.valueOf(upperBound));
        sb.append("]");

        sb.append(" created.");

        out(sb.toString());

        return attribute;
    }
    
    public static org.eclipse.uml2.uml.Class createClass(org.eclipse.uml2.uml.Package package_, String name, boolean isAbstract) {
        org.eclipse.uml2.uml.Class class_ = package_.createOwnedClass(name, isAbstract);

        out("Class '" + class_.getQualifiedName() + "' created.");

        return class_;
    }

    public static Enumeration createEnumeration(org.eclipse.uml2.uml.Package package_, String name) {
        Enumeration enumeration = (Enumeration) package_.createOwnedEnumeration(name);

        out("Enumeration '" + enumeration.getQualifiedName() + "' created.");

        return enumeration;
    }

    public static EnumerationLiteral createEnumerationLiteral(Enumeration enumeration, String name) {
        EnumerationLiteral enumerationLiteral = enumeration.createOwnedLiteral(name);

        out("Enumeration literal '" + enumerationLiteral.getQualifiedName() + "' created.");

        return enumerationLiteral;
    }    
    
    public static Extension createExtension(org.eclipse.uml2.uml.Class metaclass, Stereotype stereotype, boolean required) {
        Extension extension = stereotype.createExtension(metaclass, required);

        out((required ? "Required extension '" : "Extension '") + extension.getQualifiedName() + "' created.");

        return extension;
    }
    
    public static Generalization createGeneralization(Classifier specificClassifier, Classifier generalClassifier) {
        Generalization generalization = specificClassifier.createGeneralization(generalClassifier);

        out("Generalization " + specificClassifier.getQualifiedName() + " ->> " + generalClassifier.getQualifiedName() + " created.");

        return generalization;
    }
    
    public static Model createModel(String name) {
        Model model = UMLFactory.eINSTANCE.createModel();
        model.setName(name);

        out("Model '" + model.getQualifiedName() + "' created.");

        return model;
    }

    public static org.eclipse.uml2.uml.Package createPackage(org.eclipse.uml2.uml.Package nestingPackage, String name) {
        org.eclipse.uml2.uml.Package package_ = nestingPackage.createNestedPackage(name);

        out("Package '" + package_.getQualifiedName() + "' created.");

        return package_;
    }

    public static PrimitiveType createPrimitiveType(org.eclipse.uml2.uml.Package package_, String name) {
        PrimitiveType primitiveType = (PrimitiveType) package_.createOwnedPrimitiveType(name);

        out("Primitive type '" + primitiveType.getQualifiedName() + "' created.");

        return primitiveType;
    }    

    public static Profile createProfile(String name, String nsUri) {
        Profile profile = UMLFactory.eINSTANCE.createProfile();
        profile.setName(name);
        profile.setURI(nsUri);

        out("Profile '" + profile.getQualifiedName() + "' created.");

        return profile;
    }
    
    public static Stereotype createStereotype(Profile profile, String name, boolean isAbstract) {
        Stereotype stereotype = profile.createOwnedStereotype(name, isAbstract);

        out("Stereotype '" + stereotype.getQualifiedName() + "' created.");

        return stereotype;
    }

    public static void defineProfile(Profile profile) {
        profile.define();

        out("Profile '" + profile.getQualifiedName() + "' defined.");
    }  

    public static void err(String error) {
        System.err.println(error);
    }

    public static Object getStereotypePropertyValue(NamedElement namedElement, Stereotype stereotype, Property property) {
        Object value = namedElement.getValue(stereotype, property.getName());

        out("Value of stereotype property '" + property.getQualifiedName() + "' on element '" + namedElement.getQualifiedName()
            + "' is " + String.valueOf(value) + ".");

        return value;
    }    
    
    public static PrimitiveType importPrimitiveType(ResourceSet resourceSet, org.eclipse.uml2.uml.Package package_, String name) {
        Model umlLibrary = (Model) load(resourceSet, URI.createURI(UMLResource.UML_PRIMITIVE_TYPES_LIBRARY_URI));

        PrimitiveType primitiveType = (PrimitiveType) umlLibrary.getOwnedType(name);

        package_.createElementImport(primitiveType);

        out("Primitive type '" + primitiveType.getQualifiedName() + "' imported.");

        return primitiveType;
    }
   
    private static String resourcesLocation = null;
    
	public static void init(ResourceSet resourceSet) {
		if (resourcesLocation == null) {
			String resourcesPluginId = "org.eclipse.uml2.uml.resources";
			PluginFinder pluginFinder = new PluginFinder(resourcesPluginId);
			pluginFinder.resolve();
			resourcesLocation = pluginFinder.get(resourcesPluginId);
		}
		if ("org.eclipse.uml2.uml_3_0_0".equals(UMLResource.UML_CONTENT_TYPE_IDENTIFIER)) {
			UML300ResourcesUtil.init(resourceSet);	// MDT/UML2 3.0.0 (Helios, Indigo)
		}
		else {
			init4(resourceSet);						// MDT/UML2 4.0.0 (Juno)
		}
		Map<URI, URI> uriMap = resourceSet != null
				? resourceSet.getURIConverter().getURIMap()
				: URIConverter.URI_MAP;		
		String locationURI = resourcesLocation;
		while (locationURI.endsWith("/")) { //$NON-NLS-1$
			locationURI = locationURI.substring(0, locationURI.length()-1);
		}
		uriMap.put(URI.createURI(UMLResource.PROFILES_PATHMAP), createURI(locationURI, "/profiles/")); //$NON-NLS-1$
		uriMap.put(URI.createURI(UMLResource.METAMODELS_PATHMAP), createURI(locationURI, "/metamodels/")); //$NON-NLS-1$
		uriMap.put(URI.createURI(UMLResource.LIBRARIES_PATHMAP), createURI(locationURI, "/libraries/")); //$NON-NLS-1$
	}

    private static URI createURI(String locationURI, String string) {
		if (locationURI.endsWith(".jar!")) { //$NON-NLS-1$
			return URI.createURI(locationURI + string);
		} else {
			return URI.createFileURI(locationURI + string);
		}
	}

    private static void init4(ResourceSet resourceSet) {
    	// This is in a separate guarded routine to isolate the compilation error 
    	// from invoking a MDT/UML2 4.0.0 routine on MDT/UML2 3.0.0
    	org.eclipse.uml2.uml.resources.util.UMLResourcesUtil.init(resourceSet);
    }

    public static org.eclipse.uml2.uml.Package load(ResourceSet resourceSet, URI uri) {
		Resource resource = resourceSet.createResource(uri);

        try {
            resource.load(null);
    		org.eclipse.uml2.uml.Package package_ = (org.eclipse.uml2.uml.Package) resource.getContents().get(0);
            out("Loaded '" + package_.getQualifiedName() + "' from '" + uri + "'.");
            return package_;
        } catch (IOException ioe) {
            err(ioe.getMessage());
            return null;
        }
    }

    public static void out(String output) {

        if (DEBUG) {
            System.out.println(output);
        }
    }

    public static org.eclipse.uml2.uml.Class referenceMetaclass(ResourceSet resourceSet, Profile profile, String name) {
        Model umlMetamodel = (Model) load(resourceSet, URI.createURI(UMLResource.UML_METAMODEL_URI));

        org.eclipse.uml2.uml.Class metaclass = (org.eclipse.uml2.uml.Class) umlMetamodel.getOwnedType(name);

        profile.createMetaclassReference(metaclass);

        out("Metaclass '" + metaclass.getQualifiedName() + "' referenced.");

        return metaclass;
    }
    
    public static void save(ResourceSet resourceSet, org.eclipse.uml2.uml.Package package_, URI uri) {
		Resource resource = resourceSet.createResource(uri);
        resource.getContents().add(package_);

        try {
            resource.save(null);
            out("Saved '" + package_.getQualifiedName() + "' as '" + uri + "'.");
        } catch (IOException ioe) {
            err(ioe.getMessage());
        }
    }
    
    public static void setStereotypePropertyValue(NamedElement namedElement, Stereotype stereotype, Property property, Object value) {
        namedElement.setValue(stereotype, property.getName(), value);

        out("Value of stereotype property '" + property.getQualifiedName() + "' on element '" + namedElement.getQualifiedName()
            + "' set to " + String.valueOf(value) + ".");
    }

}
