/*
 * Copyright (c) 2012 IBM and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   E.D.Willink - complete code for fragmentary Wiki example.
 *
 */
package org.eclipse.uml2.uml.examples.standalone;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UMLResource;

/**
 * This class provides a working example of
 * http://wiki.eclipse.org/MDT/UML2/Introduction_to_UML2_Profiles.
 */
public class IntroductionToUMLProfilesExample extends GettingStartedExample
{
    @SuppressWarnings("unused")
	public static void main(String[] args) {
    	if (args.length != 1) {
    		err("Usage: <results-folder>");
    		return;
    	}
        //
        //	Init
        //
    	ResourceSet resourceSet = new ResourceSetImpl();
    	init(resourceSet);
        //
        //	Create
        //
        Profile ecoreProfile = createProfile("ecore", EcorePackage.eNS_URI);    	
        PrimitiveType booleanPrimitiveType = importPrimitiveType(resourceSet, ecoreProfile, "Boolean");    	
        PrimitiveType stringPrimitiveType = importPrimitiveType(resourceSet, ecoreProfile, "String");    	
    	
        Enumeration visibilityKindEnumeration = createEnumeration(ecoreProfile, "VisibilityKind");
        Enumeration featureKindEnumeration = createEnumeration(ecoreProfile, "FeatureKind");
        
        createEnumerationLiteral(visibilityKindEnumeration, "Unspecified");
        createEnumerationLiteral(visibilityKindEnumeration, "None");
        createEnumerationLiteral(visibilityKindEnumeration, "ReadOnly");
        createEnumerationLiteral(visibilityKindEnumeration, "ReadWrite");
        createEnumerationLiteral(visibilityKindEnumeration, "ReadOnlyUnsettable");
        createEnumerationLiteral(visibilityKindEnumeration, "ReadWriteUnsettable");
        createEnumerationLiteral(featureKindEnumeration, "Unspecified");
        createEnumerationLiteral(featureKindEnumeration, "Simple");
        createEnumerationLiteral(featureKindEnumeration, "Attribute");
        createEnumerationLiteral(featureKindEnumeration, "Element");
        createEnumerationLiteral(featureKindEnumeration, "AttributeWildcard");
        createEnumerationLiteral(featureKindEnumeration, "ElementWildcard");
        createEnumerationLiteral(featureKindEnumeration, "Group");
  
        Stereotype eStructuralFeatureStereotype = createStereotype(ecoreProfile, "EStructuralFeature", true);        
        Stereotype eAttributeStereotype = createStereotype(ecoreProfile, "EAttribute", true);        
        Stereotype eReferenceStereotype = createStereotype(ecoreProfile, "EReference", true);        

        createGeneralization(eAttributeStereotype, eStructuralFeatureStereotype);
        createGeneralization(eReferenceStereotype, eStructuralFeatureStereotype);

        Property isTransientProperty = createAttribute(eStructuralFeatureStereotype, "isTransient", booleanPrimitiveType, 0, 1);    
        Property isUnsettableProperty = createAttribute(eStructuralFeatureStereotype, "isUnsettable", booleanPrimitiveType, 0, 1);    
        Property isVolatileProperty = createAttribute(eStructuralFeatureStereotype, "isVolatile", booleanPrimitiveType, 0, 1);    
        Property visibilityProperty = createAttribute(eStructuralFeatureStereotype, "visibility", visibilityKindEnumeration, 0, 1);    
        Property xmlNameProperty = createAttribute(eStructuralFeatureStereotype, "xmlName", stringPrimitiveType, 0, 1);    
        Property xmlNamespaceProperty = createAttribute(eStructuralFeatureStereotype, "xmlNamespace", stringPrimitiveType, 0, 1);    
        Property xmlFeatureKindProperty = createAttribute(eStructuralFeatureStereotype, "xmlFeatureKind", featureKindEnumeration, 0, 1);    
        Property attributeNameProperty = createAttribute(eAttributeStereotype, "attributeName", stringPrimitiveType, 0, 1);    
        Property referenceNameProperty = createAttribute(eReferenceStereotype, "referenceName", stringPrimitiveType, 0, 1);
        Property isResolveProxiesProperty = createAttribute(eReferenceStereotype, "isResolveProxies", booleanPrimitiveType, 0, 1);
 
        org.eclipse.uml2.uml.Class propertyMetaclass = referenceMetaclass(resourceSet, ecoreProfile, UMLPackage.Literals.PROPERTY.getName());        

        createExtension(propertyMetaclass, eAttributeStereotype, false);
        createExtension(propertyMetaclass, eReferenceStereotype, false);
        
        //
        //	Save profile
        //
        URI folderURI = URI.createFileURI(args[0]);
		URI profileURI = folderURI.appendSegment("Ecore").appendFileExtension(UMLResource.PROFILE_FILE_EXTENSION);
		save(resourceSet, ecoreProfile, profileURI);     

		//
        //	Load model
        //
		URI fileURI = folderURI.appendSegment("ExtendedPO2").appendFileExtension(UMLResource.FILE_EXTENSION);
		Model epo2Model = (Model) load(resourceSet, fileURI);     
    	
        applyProfile(epo2Model, ecoreProfile);    	

        org.eclipse.uml2.uml.Class customerClass = (org.eclipse.uml2.uml.Class) epo2Model.getOwnedType("Customer");
        org.eclipse.uml2.uml.Class purchaseOrderClass = (org.eclipse.uml2.uml.Class) epo2Model.getOwnedType("PurchaseOrder");
        org.eclipse.uml2.uml.Class supplierClass = (org.eclipse.uml2.uml.Class) epo2Model.getOwnedType("Supplier");
        
        Property customerIDProperty = customerClass.getOwnedAttribute("customerID", null);
//        Property customerProperty = purchaseOrderClass.getOwnedAttribute("customer", null);
//        Property ordersProperty = customerClass.getOwnedAttribute("orders", null);
//        Property pendingOrdersProperty = supplierClass.getOwnedAttribute("pendingOrders", null);
//        Property previousOrderProperty = purchaseOrderClass.getOwnedAttribute("previousOrder", null);
//        Property shippedOrdersProperty = supplierClass.getOwnedAttribute("shippedOrders", null);
//        Property totalAmountProperty = purchaseOrderClass.getOwnedAttribute("totalAmount", null);

        applyStereotype(customerIDProperty, eAttributeStereotype);
// FIXME these don't work
//        applyStereotype(customerProperty, eReferenceStereotype);
//        applyStereotype(ordersProperty, eReferenceStereotype);
//        applyStereotype(pendingOrdersProperty, eReferenceStereotype);
//        applyStereotype(previousOrderProperty, eReferenceStereotype);
//        applyStereotype(shippedOrdersProperty, eReferenceStereotype);
//        applyStereotype(totalAmountProperty, eReferenceStereotype);

// FIXME this gives an IAE see Bug 382342
        Boolean isVolatile = (Boolean) getStereotypePropertyValue(customerIDProperty, eAttributeStereotype, isVolatileProperty);

        setStereotypePropertyValue(customerIDProperty, eAttributeStereotype, isVolatileProperty, Boolean.TRUE);
        
// FIXME Add profiled save  
    }
}